package core_profiler

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Core_profiler_offline_tool

module Event_generator : sig ... end
module Filter : sig ... end
module Id_table : sig ... end

An Id_table.t exploits the fact that Probe_id.ts should be consecutive integers to make lookups fast

module Interest : sig ... end
module Path : sig ... end

The first and last points are stored explicitly, not least to ensure that there are two of them. The list of points in the middle is stored in reverse order for convenience when checking the path.

module Reader : sig ... end
module Reservoir_sampling : sig ... end
module Std : sig ... end
module Util : sig ... end