package core_profiler

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Core_profiler_offline_tool.FilterSource

Sourcetype -'rw t
Sourceval create : Reader.Header.t -> [< _ Core.perms ] t
Sourceval read_only : [> Core.read ] t -> Core.read t

Which interests does this event match?

Sourceval test' : [> Core.read ] t -> Event_generator.event -> bool

Does this event match anything?

Sourceval iter_events : [> Core.read ] t -> Event_generator.t -> f:(Event_generator.event -> Core_profiler.Probe_id.t Interest.t list -> unit) -> unit

f is given the event, and the interests it matched

Sourceval iter_events_interests : [> Core.read ] t -> Event_generator.t -> f:(Event_generator.event -> Core_profiler.Probe_id.t Interest.t -> unit) -> unit
Sourceval raw_interests : [> Core.read ] t -> Core_profiler.Probe_id.t Interest.Raw.t list

All raw interests the filter is (currently) interested in receiving events for. In the basic case, you'll want to construct a t (a filter), and then build a Event_generator.t using the raw interests specified by this function.