package core_profiler

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Reader.HeaderSource

Sourcemodule Item : sig ... end

A Header.Item.t is anything that is uniquely identified by a Probe_id.t.

Sourceval find_single_exn : t -> Core_profiler.Probe_id.t -> Item.single
Sourceval find_group_exn : t -> Core_profiler.Probe_id.t -> Item.group
Sourceval find_group_point_exn : t -> Core_profiler.Probe_id.t -> Item.group_point

Get a group point's parent

Sourceval get_parent_exn : t -> Core_profiler.Probe_id.t -> Item.group
Sourceval get_name_exn : t -> ?with_group:string -> Core_profiler.Probe_id.t -> string

If add_group is specified and the id refers to a group point, group_name ^ add_group ^ group_point_name is returned

Sourceval create_table : t -> ?singles:bool -> ?groups:bool -> ?group_points:bool -> ?timers:bool -> ?probes:bool -> 'a -> ('a, _) Id_table.t

Conditions are ANDed, and default to true