package core_profiler

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Header_protocol.New_groupSource

Sourcetype phantom = [
  1. | `New_group
]
Sourcetype nonrec -'rw t = (phantom, 'rw) t constraint 'rw = [> Core.read ]
Sourceval sexp_of_t : ('rw -> Ppx_sexp_conv_lib.Sexp.t) -> 'rw t -> Ppx_sexp_conv_lib.Sexp.t
Sourceval t_of_sexp : (Ppx_sexp_conv_lib.Sexp.t -> 'rw) -> Ppx_sexp_conv_lib.Sexp.t -> 'rw t
Sourceval message_type : char
Sourceval buffer_length : int
Sourceval of_iobuf_exn : ('rw, _) Iobuf.t -> 'rw t
Sourceval write : id:Probe_id.t -> spec:Probe_type.t -> name:string -> (Core.read_write, _) Iobuf.t -> int
Sourceval create : id:Probe_id.t -> spec:Probe_type.t -> name:string -> (Core.read_write, Iobuf.seek) Iobuf.t
Sourceval get_message_length : _ t -> int
Sourceval get_message_type : _ t -> char
Sourceval get_id : _ t -> Probe_id.t
Sourceval get_spec : _ t -> Probe_type.t
Sourceval name_max_len : int
Sourceval get_name : _ t -> string
Sourceval get_name_zero : _ t -> ((Core.read, Iobuf.no_seek) Iobuf.t -> safe_pos:int -> safe_len:int -> 'a) -> 'a
Sourceval get_name_zero_padded : _ t -> ((Core.read, Iobuf.no_seek) Iobuf.t -> safe_pos:int -> safe_len:int -> 'a) -> 'a
Sourceval set_id : (Core.read_write, _) Iobuf.t -> Probe_id.t -> unit
Sourceval set_spec : (Core.read_write, _) Iobuf.t -> Probe_type.t -> unit
Sourceval set_name : (Core.read_write, _) Iobuf.t -> string -> unit
Sourceval set_name_zero : (Core.read_write, Iobuf.seek) Iobuf.t -> ('a -> (Core.read_write, Iobuf.seek) Iobuf.t -> unit) -> 'a -> unit

set_name_zero buf f a calls f on buf, with the window adjusted to where name is. Even though f is given a seekable buffer, it must move nothing except the lower bound of the window past the data it wrote.

Sourceval to_sub_iobuf : 'rw t -> ('rw, Iobuf.seek) Iobuf.t
Sourcemodule Unpacked : sig ... end
Sourceval to_unpacked : 'rw t -> Unpacked.t
Sourceval of_unpacked : Unpacked.t -> 'rw t