package core_profiler

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Online.Delta_timerSource

Delta_timer is an optimized two-probe group to track time differences between calls to start and stop.

Sourcetype t
Sourcetype state = private Core.Time_ns.t
Sourceval create : name:string -> t
Sourceval start : t -> unit
Sourceval stop : t -> unit
Sourceval pause : t -> unit
Sourceval record : t -> unit
Sourceval stateless_start : t -> state
Sourceval stateless_stop : t -> state -> unit
Sourceval wrap_sync : t -> ('a -> 'b) -> 'a -> 'b

Typically partially applied (the first two arguments) to produce a 'wrapped' function. This behaves like the identity function on functions, except it times the inner function.

Sourceval wrap_sync2 : t -> ('a -> 'b -> 'c) -> 'a -> 'b -> 'c
Sourceval wrap_sync3 : t -> ('a -> 'b -> 'c -> 'd) -> 'a -> 'b -> 'c -> 'd
Sourceval wrap_sync4 : t -> ('a -> 'b -> 'c -> 'd -> 'e) -> 'a -> 'b -> 'c -> 'd -> 'e