package core_profiler

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Protocol.WriterSource

The Writer module contains functions that invoke parts of Header_protocol and Short_header in order to write into the relevant (global variable) buffers in Buffer

Sourceval max_time : Core.Time_ns.t
Sourceval write_new_single : Probe_id.t -> string -> Probe_type.t -> unit

These write into the header chunk

Sourceval write_new_group : Probe_id.t -> string -> Probe_type.t -> unit
Sourceval write_new_group_point : group_id:Probe_id.t -> id:Probe_id.t -> string -> Probe_id.t array -> unit
Sourceval write_timer_at : Probe_id.t -> Core.Time_ns.t -> unit

These write into the short message buffer

Sourceval write_probe_at : Probe_id.t -> Core.Time_ns.t -> int -> unit
Sourceval write_group_reset : Probe_id.t -> Core.Time_ns.t -> unit
Sourceval dump_stats : unit -> unit
Sourceval set_at_exit_handler : [ `Write_file of string | `Function of (Core.read, Iobuf.no_seek) Iobuf.t -> (Core.read, Iobuf.no_seek) Iobuf.t list -> unit | `Disable ] -> unit

Choose what to do with the in memory stats data at exit. The handler function is passed the header chunk and the list of data chunks. Defaults to `Write_file "stats.dat".

Sourcemodule Unsafe_internals : sig ... end

To aid producing test cases for Reader.